from telebox.dispatcher.filters.event_filter import AbstractEventFilter
from telebox.dispatcher.enums.event_type import EventType
from telebox.dispatcher.utils.media_group import MediaGroup
from telebox.dispatcher.enums.media_group_content_type import MediaGroupContentType


class MediaGroupContentTypeFilter(AbstractEventFilter):

    def __init__(self, *types: MediaGroupContentType):
        if not types:
            raise ValueError("No media group content types!")

        self._types = set(types)

    def get_event_types(self) -> set[EventType]:
        return {EventType.MEDIA_GROUP}

    def get_value(self, event: MediaGroup) -> set[MediaGroupContentType]:
        return event.content_types

    def check_value(self, value: set[MediaGroupContentType]) -> bool:
        return any(i in value for i in self._types)



from dataclasses import dataclass
from datetime import datetime
from typing import Optional, Literal, Union, TYPE_CHECKING

from telebox.bot.consts import chat_types
from telebox.bot.utils.deep_links import get_message_public_link, get_message_private_link
from telebox.bot.enums.message_content_type import MessageContentType
from telebox.bot.errors import UnknownMessageContentError
from telebox.bot.types.type import Type
from telebox.bot.types.types.user import User
from telebox.bot.types.types.message_entity import MessageEntity
from telebox.bot.types.types.animation import Animation
from telebox.bot.types.types.audio import Audio
from telebox.bot.types.types.document import Document
from telebox.bot.types.types.photo_size import PhotoSize
from telebox.bot.types.types.sticker import Sticker
from telebox.bot.types.types.video import Video
from telebox.bot.types.types.video_note import VideoNote
from telebox.bot.types.types.voice import Voice
from telebox.bot.types.types.contact import Contact
from telebox.bot.types.types.dice import Dice
from telebox.bot.types.types.game import Game
from telebox.bot.types.types.poll import Poll
from telebox.bot.types.types.venue import Venue
from telebox.bot.types.types.location import Location
from telebox.bot.types.types.invoice import Invoice
from telebox.bot.types.types.successful_payment import SuccessfulPayment
from telebox.bot.types.types.passport_data import PassportData
from telebox.bot.types.types.proximity_alert_triggered import ProximityAlertTriggered
from telebox.bot.types.types.forum_topic_created import ForumTopicCreated
from telebox.bot.types.types.forum_topic_closed import ForumTopicClosed
from telebox.bot.types.types.forum_topic_reopened import ForumTopicReopened
from telebox.bot.types.types.video_chat_scheduled import VideoChatScheduled
from telebox.bot.types.types.video_chat_started import VideoChatStarted
from telebox.bot.types.types.video_chat_ended import VideoChatEnded
from telebox.bot.types.types.web_app_data import WebAppData
from telebox.bot.types.types.inline_keyboard_markup import InlineKeyboardMarkup
from telebox.bot.types.types.message_auto_delete_timer_changed import (
    MessageAutoDeleteTimerChanged
)
from telebox.bot.types.types.video_chat_participants_invited import (
    VideoChatParticipantsInvited
)
if TYPE_CHECKING:
    from telebox.bot.types.types.chat import Chat


@dataclass(unsafe_hash=True)
class Message(Type):
    message_id: int
    date: datetime
    chat: "Chat"
    message_thread_id: Optional[int] = None
    from_: Optional[User] = None
    sender_chat: Optional["Chat"] = None
    forward_from: Optional[User] = None
    forward_from_chat: Optional["Chat"] = None
    forward_from_message_id: Optional[int] = None
    forward_signature: Optional[str] = None
    forward_sender_name: Optional[str] = None
    forward_date: Optional[datetime] = None
    is_topic_message: Optional[Literal[True]] = None
    is_automatic_forward: Optional[Literal[True]] = None
    reply_to_message: Optional["Message"] = None
    via_bot: Optional[User] = None
    edit_date: Optional[datetime] = None
    has_protected_content: Optional[Literal[True]] = None
    media_group_id: Optional[str] = None
    author_signature: Optional[str] = None
    text: Optional[str] = None
    entities: Optional[list[MessageEntity]] = None
    animation: Optional[Animation] = None
    audio: Optional[Audio] = None
    document: Optional[Document] = None
    photo: Optional[list[PhotoSize]] = None
    sticker: Optional[Sticker] = None
    video: Optional[Video] = None
    video_note: Optional[VideoNote] = None
    voice: Optional[Voice] = None
    caption: Optional[str] = None
    caption_entities: Optional[list[MessageEntity]] = None
    contact: Optional[Contact] = None
    dice: Optional[Dice] = None
    game: Optional[Game] = None
    poll: Optional[Poll] = None
    venue: Optional[Venue] = None
    location: Optional[Location] = None
    new_chat_members: Optional[list[User]] = None
    left_chat_member: Optional[User] = None
    new_chat_title: Optional[str] = None
    new_chat_photo: Optional[list[PhotoSize]] = None
    delete_chat_photo: Optional[Literal[True]] = None
    group_chat_created: Optional[Literal[True]] = None
    supergroup_chat_created: Optional[Literal[True]] = None
    channel_chat_created: Optional[Literal[True]] = None
    message_auto_delete_timer_changed: Optional[MessageAutoDeleteTimerChanged] = None
    migrate_to_chat_id: Optional[int] = None
    migrate_from_chat_id: Optional[int] = None
    pinned_message: Optional["Message"] = None
    invoice: Optional[Invoice] = None
    successful_payment: Optional[SuccessfulPayment] = None
    connected_website: Optional[str] = None
    passport_data: Optional[PassportData] = None
    proximity_alert_triggered: Optional[ProximityAlertTriggered] = None
    forum_topic_created: Optional[ForumTopicCreated] = None
    forum_topic_closed: Optional[ForumTopicClosed] = None
    forum_topic_reopened: Optional[ForumTopicReopened] = None
    video_chat_scheduled: Optional[VideoChatScheduled] = None
    video_chat_started: Optional[VideoChatStarted] = None
    video_chat_ended: Optional[VideoChatEnded] = None
    video_chat_participants_invited: Optional[VideoChatParticipantsInvited] = None
    web_app_data: Optional[WebAppData] = None
    reply_markup: Optional[InlineKeyboardMarkup] = None

    @property
    def content(self) -> tuple["MessageContent", MessageContentType]:
        if self.text is not None:
            return self.text, MessageContentType.TEXT
        elif self.animation is not None:
            return self.animation, MessageContentType.ANIMATION
        elif self.audio is not None:
            return self.audio, MessageContentType.AUDIO
        elif self.document is not None:
            return self.document, MessageContentType.DOCUMENT
        elif self.photo is not None:
            return self.photo, MessageContentType.PHOTO
        elif self.sticker is not None:
            return self.sticker, MessageContentType.STICKER
        elif self.video is not None:
            return self.video, MessageContentType.VIDEO
        elif self.video_note is not None:
            return self.video_note, MessageContentType.VIDEO_NOTE
        elif self.voice is not None:
            return self.voice, MessageContentType.VOICE
        elif self.contact is not None:
            return self.contact, MessageContentType.CONTACT
        elif self.dice is not None:
            return self.dice, MessageContentType.DICE
        elif self.game is not None:
            return self.game, MessageContentType.GAME
        elif self.poll is not None:
            return self.poll, MessageContentType.POLL
        elif self.venue is not None:
            return self.venue, MessageContentType.VENUE
        elif self.location is not None:
            return self.location, MessageContentType.LOCATION
        elif self.new_chat_members is not None:
            return self.new_chat_members, MessageContentType.NEW_CHAT_MEMBERS
        elif self.left_chat_member is not None:
            return self.left_chat_member, MessageContentType.LEFT_CHAT_MEMBER
        elif self.new_chat_title is not None:
            return self.new_chat_title, MessageContentType.NEW_CHAT_TITLE
        elif self.new_chat_photo is not None:
            return self.new_chat_photo, MessageContentType.NEW_CHAT_PHOTO
        elif self.delete_chat_photo is not None:
            return self.delete_chat_photo, MessageContentType.DELETE_CHAT_PHOTO
        elif self.group_chat_created is not None:
            return self.group_chat_created, MessageContentType.GROUP_CHAT_CREATED
        elif self.supergroup_chat_created is not None:
            return self.supergroup_chat_created, MessageContentType.SUPERGROUP_CHAT_CREATED
        elif self.channel_chat_created is not None:
            return self.channel_chat_created, MessageContentType.CHANNEL_CHAT_CREATED
        elif self.message_auto_delete_timer_changed is not None:
            return (
                self.message_auto_delete_timer_changed,
                MessageContentType.MESSAGE_AUTO_DELETE_TIMER_CHANGED
            )
        elif self.migrate_to_chat_id is not None:
            return self.migrate_to_chat_id, MessageContentType.MIGRATE_TO_CHAT_ID
        elif self.migrate_from_chat_id is not None:
            return self.migrate_from_chat_id, MessageContentType.MIGRATE_FROM_CHAT_ID
        elif self.pinned_message is not None:
            return self.pinned_message, MessageContentType.PINNED_MESSAGE
        elif self.invoice is not None:
            return self.invoice, MessageContentType.INVOICE
        elif self.successful_payment is not None:
            return self.successful_payment, MessageContentType.SUCCESSFUL_PAYMENT
        elif self.connected_website is not None:
            return self.connected_website, MessageContentType.CONNECTED_WEBSITE
        elif self.passport_data is not None:
            return self.passport_data, MessageContentType.PASSPORT_DATA
        elif self.proximity_alert_triggered is not None:
            return self.proximity_alert_triggered, MessageContentType.PROXIMITY_ALERT_TRIGGERED
        elif self.forum_topic_created is not None:
            return self.forum_topic_created, MessageContentType.FORUM_TOPIC_CREATED
        elif self.forum_topic_closed is not None:
            return self.forum_topic_closed, MessageContentType.FORUM_TOPIC_CLOSED
        elif self.forum_topic_reopened is not None:
            return self.forum_topic_reopened, MessageContentType.FORUM_TOPIC_REOPENED
        elif self.video_chat_scheduled is not None:
            return self.video_chat_scheduled, MessageContentType.VIDEO_CHAT_SCHEDULED
        elif self.video_chat_started is not None:
            return self.video_chat_started, MessageContentType.VIDEO_CHAT_STARTED
        elif self.video_chat_ended is not None:
            return self.video_chat_ended, MessageContentType.VIDEO_CHAT_ENDED
        elif self.video_chat_participants_invited is not None:
            return (
                self.video_chat_participants_invited,
                MessageContentType.VIDEO_CHAT_PARTICIPANTS_INVITED
            )
        elif self.web_app_data is not None:
            return self.web_app_data, MessageContentType.WEB_APP_DATA

        raise UnknownMessageContentError("Unknown message type!")

    @property
    def chat_type(self) -> str:
        return self.chat.type

    @property
    def chat_id(self) -> int:
        return self.chat.id

    @property
    def sender_chat_id(self) -> Optional[int]:
        return self.sender_chat.id if self.sender_chat is not None else None

    @property
    def user_id(self) -> Optional[int]:
        return self.from_.id if self.from_ is not None else None

    @property
    def is_forwarded(self) -> bool:
        return self.forward_date is not None

    @property
    def is_reply(self) -> bool:
        return self.reply_to_message is not None

    @property
    def link(self) -> Optional[str]:
        if self.chat.type in {chat_types.CHANNEL, chat_types.SUPERGROUP}:
            if self.chat.username is not None:
                return get_message_public_link(self.chat.username, self.message_id)
            else:
                return get_message_private_link(self.chat.id, self.message_id)

    def get_text(self) -> Optional[str]:
        if self.text is not None:
            return self.text
        elif self.caption is not None:
            return self.caption

    def get_entity_text(self, entity: MessageEntity) -> Optional[str]:
        text = self.get_text()

        if text is not None:
            bytes_ = text.encode("UTF-16-LE")
            entity_bytes = bytes_[entity.offset * 2:entity.end_offset * 2]

            return entity_bytes.decode("UTF-16-LE")

    def get_entities(self) -> list[MessageEntity]:
        if self.entities is not None:
            return self.entities
        elif self.caption_entities is not None:
            return self.caption_entities

        return []

    def get_command_args(self) -> list[str]:
        text = self.get_text()
        args = []

        for i in text.split(" ")[1:]:
            if i:
                args.append(i)

        return args


MessageContent = Union[
    str,
    Animation,
    Audio,
    Document,
    list[PhotoSize],
    Sticker,
    Video,
    VideoNote,
    Voice,
    Contact,
    Dice,
    Game,
    Poll,
    Venue,
    Location,
    list[User],
    User,
    Literal[True],
    MessageAutoDeleteTimerChanged,
    int,
    Message,
    Invoice,
    SuccessfulPayment,
    PassportData,
    ProximityAlertTriggered,
    ForumTopicCreated,
    ForumTopicClosed,
    ForumTopicReopened,
    VideoChatScheduled,
    VideoChatStarted,
    VideoChatEnded,
    VideoChatParticipantsInvited,
    WebAppData
]





class NotSet:
    __instance = None

    def __new__(cls):
        if cls.__instance is None:
            cls.__instance = super().__new__(cls)

        return cls.__instance

    def __repr__(self):
        return type(self).__name__

    def __bool__(self):
        return False


NOT_SET = NotSet()



from .none import NoneFilter
from .command import CommandFilter
from .text import TextFilter
from .reg_exp_text import RegExpTextFilter
from .message_content_type import MessageContentTypeFilter
from .sender_contact import SenderContactFilter
from .chat_id import ChatIDFilter
from .user_id import UserIDFilter
from .reply_message import ReplyMessageFilter
from .forwarded_message import ForwardedMessageFilter
from .chat_type import ChatTypeFilter
from .state import StateFilter
from .non_standard_command import NonStandardCommandFilter
from .callback_data import CallbackDataFilter
from .simple_callback_data import SimpleCallbackDataFilter
from .hashtag import HashtagFilter
from .cashtag import CashtagFilter
from .mention import MentionFilter
from .text_mention import TextMentionFilter
from .media_group_content_type import MediaGroupContentTypeFilter
from .new_chat_member_id import NewChatMemberIDFilter


__all__ = [
    "NoneFilter",
    "CommandFilter",
    "TextFilter",
    "RegExpTextFilter",
    "MessageContentTypeFilter",
    "SenderContactFilter",
    "ChatIDFilter",
    "UserIDFilter",
    "ReplyMessageFilter",
    "ForwardedMessageFilter",
    "ChatTypeFilter",
    "StateFilter",
    "NonStandardCommandFilter",
    "CallbackDataFilter",
    "SimpleCallbackDataFilter",
    "HashtagFilter",
    "CashtagFilter",
    "MentionFilter",
    "TextMentionFilter",
    "MediaGroupContentTypeFilter",
    "NewChatMemberIDFilter"
]



from dataclasses import dataclass
from typing import Optional

from telebox.bot.types.type import Type


@dataclass(unsafe_hash=True)
class ForumTopic(Type):
    message_thread_id: int
    name: str
    icon_color: int
    icon_custom_emoji_id: Optional[str] = None



from typing import Union

from telebox.bot.types.types.passport_element_error_data_field import (
    PassportElementErrorDataField
)
from telebox.bot.types.types.passport_element_error_front_side import (
    PassportElementErrorFrontSide
)
from telebox.bot.types.types.passport_element_error_reverse_side import (
    PassportElementErrorReverseSide
)
from telebox.bot.types.types.passport_element_error_selfie import (
    PassportElementErrorSelfie
)
from telebox.bot.types.types.passport_element_error_file import (
    PassportElementErrorFile
)
from telebox.bot.types.types.passport_element_error_files import (
    PassportElementErrorFiles
)
from telebox.bot.types.types.passport_element_error_translation_file import (
    PassportElementErrorTranslationFile
)
from telebox.bot.types.types.passport_element_error_translation_files import (
    PassportElementErrorTranslationFiles
)
from telebox.bot.types.types.passport_element_error_unspecified import (
    PassportElementErrorUnspecified
)


PassportElementError = Union[PassportElementErrorDataField,
                             PassportElementErrorFrontSide,
                             PassportElementErrorReverseSide,
                             PassportElementErrorSelfie,
                             PassportElementErrorFile,
                             PassportElementErrorFiles,
                             PassportElementErrorTranslationFile,
                             PassportElementErrorTranslationFiles,
                             PassportElementErrorUnspecified]



from abc import ABC, abstractmethod
from typing import Optional, Any

from telebox.dispatcher.enums.event_type import EventType


class AbstractEventBaseFilter(ABC):

    def __invert__(self):
        return InvertEventFilter(self)

    def __and__(self, other):
        if isinstance(other, ConjunctionEventFilter):
            return ConjunctionEventFilter(self, *other)
        elif isinstance(other, AbstractEventBaseFilter):
            return ConjunctionEventFilter(self, other)

        return NotImplemented

    def __or__(self, other):
        if isinstance(other, DisjunctionEventFilter):
            return DisjunctionEventFilter(self, *other)
        elif isinstance(other, AbstractEventBaseFilter):
            return DisjunctionEventFilter(self, other)

        return NotImplemented

    @abstractmethod
    def get_event_types(self) -> set[EventType]:
        pass

    @abstractmethod
    def check_event_type(self, event_type: EventType) -> bool:
        pass

    @abstractmethod
    def get_result(self, event, values: Optional[dict[type, Any]] = None) -> bool:
        pass


class AbstractEventFilter(AbstractEventBaseFilter, ABC):

    def check_event_type(self, event_type: EventType) -> bool:
        return event_type in self.get_event_types()

    @abstractmethod
    def get_value(self, event):
        pass

    @abstractmethod
    def check_value(self, value) -> bool:
        pass

    def get_result(self, event, values: Optional[dict[type, Any]] = None) -> bool:
        values = values or {}

        try:
            value = values[type(self)]
        except KeyError:
            value = values[type(self)] = self.get_value(event)

        return self.check_value(value)


class InvertEventFilter(AbstractEventBaseFilter):

    def __init__(self, filter_: AbstractEventBaseFilter):
        self.filter = filter_

    def __repr__(self):
        return f"{type(self).__name__}({self.filter!r})"

    def __invert__(self):
        return self.filter

    def get_event_types(self) -> set[EventType]:
        return self.filter.get_event_types()

    def check_event_type(self, event_type: EventType) -> bool:
        return event_type in self.filter.get_event_types()

    def get_result(self, event, values: Optional[dict[type, Any]] = None) -> bool:
        return not self.filter.get_result(event, values)


class ConjunctionEventFilter(AbstractEventBaseFilter):

    def __init__(self, *filters: AbstractEventBaseFilter):
        self.filters = filters

    def __repr__(self):
        return f"{type(self).__name__}({', '.join(repr(i) for i in self.filters)})"

    def __iter__(self):
        return iter(self.filters)

    def __and__(self, other):
        if isinstance(other, ConjunctionEventFilter):
            return ConjunctionEventFilter(*self, *other)
        elif isinstance(other, AbstractEventBaseFilter):
            return ConjunctionEventFilter(*self, other)

        return NotImplemented

    def get_event_types(self) -> set[EventType]:
        types = set()

        for i in self.filters:
            types.update(i.get_event_types())

        return types

    def check_event_type(self, event_type: EventType) -> bool:
        for i in self.filters:
            if event_type not in i.get_event_types():
                return False

        return True

    def get_result(self, event, values: Optional[dict[type, Any]] = None) -> bool:
        return all(i.get_result(event, values) for i in self.filters)


class DisjunctionEventFilter(AbstractEventBaseFilter):

    def __init__(self, *filters: AbstractEventBaseFilter):
        self.filters = filters

    def __repr__(self):
        return f"{type(self).__name__}({', '.join(repr(i) for i in self.filters)})"

    def __iter__(self):
        return iter(self.filters)

    def __or__(self, other):
        if isinstance(other, DisjunctionEventFilter):
            return DisjunctionEventFilter(*self, *other)
        elif isinstance(other, AbstractEventBaseFilter):
            return DisjunctionEventFilter(*self, other)

        return NotImplemented

    def get_event_types(self) -> set[EventType]:
        types = set()

        for i in self.filters:
            types.update(i.get_event_types())

        return types

    def check_event_type(self, event_type: EventType) -> bool:
        for i in self.filters:
            if event_type not in i.get_event_types():
                return False

        return True

    def get_result(self, event, values: Optional[dict[type, Any]] = None) -> bool:
        return any(i.get_result(event, values) for i in self.filters)



from typing import Optional, Union

from telebox.dispatcher.filters.event_filter import AbstractEventFilter
from telebox.dispatcher.enums.event_type import EventType
from telebox.dispatcher.utils.media_group import MediaGroup
from telebox.bot.types.types.message import Message


class NonStandardCommandFilter(AbstractEventFilter):

    def __init__(self, *commands: str, prefix: str, ignore_case: bool = True):
        self._commands = set()

        for i in commands:
            if not i.startswith(prefix):
                i = f"{prefix}{i}"

            if ignore_case:
                i = i.lower()

            self._commands.add(i)

        self._prefix = prefix
        self._ignore_case = ignore_case

    def get_event_types(self) -> set[EventType]:
        return {
            EventType.MESSAGE,
            EventType.EDITED_MESSAGE,
            EventType.CHANNEL_POST,
            EventType.EDITED_CHANNEL_POST,
            EventType.MEDIA_GROUP
        }

    def get_value(self, event: Union[Message, MediaGroup]) -> Optional[str]:
        if isinstance(event, MediaGroup):
            for i in event:
                command = _get_command(i)

                if command is not None:
                    return command

        return _get_command(event)

    def check_value(self, value: Optional[str]) -> bool:
        if (value is not None) and value.startswith(self._prefix):
            if not self._commands:
                return True

            if self._ignore_case:
                value = value.lower()

            return value in self._commands

        return False


def _get_command(message: Message) -> Optional[str]:
    text = message.get_text()

    if text is not None:
        return text.split(" ", 1)[0]



from typing import Any
import time


class TimedContainer:

    def __init__(self, initial_item: Any):
        self.items = [initial_item]
        self.time = time.monotonic()

    def add(self, item: Any) -> None:
        self.items.append(item)
        self.time = time.monotonic()



from typing import Optional


class StateMagazine:

    def __init__(self, states: list[str]):
        if not states:
            raise ValueError("State magazine cannot be empty!")

        self._states = states[:]

    def __iter__(self):
        return iter(self._states)

    def __repr__(self):
        return f"{type(self).__name__}({self._states!r})"

    @property
    def current_state(self) -> str:
        return self._states[-1]

    @property
    def previous_state(self) -> Optional[str]:
        try:
            return self._states[-2]
        except IndexError:
            return None

    def set_state(self, state: str) -> None:
        try:
            index = self._states.index(state)
        except ValueError:
            self._states.append(state)
        else:
            del self._states[index + 1:]



from dataclasses import dataclass
import dataclasses
from typing import Iterable, Any


@dataclass
class NamedSet:

    def __post_init__(self):
        fields = {i.name for i in dataclasses.fields(self)}
        self._items = {
            value
            for name, value in vars(self).items()
            if name in fields
        }

    def __len__(self):
        return len(self._items)

    def __bool__(self):
        return bool(self._items)

    def __iter__(self):
        return iter(self._items)

    def __repr__(self):
        return f"{type(self).__name__}({self._items!r})"

    def __contains__(self, item):
        return item in self._items

    def get(self, *, exclude: Iterable[Any] = ()) -> set[Any]:
        return {i for i in self._items if i not in exclude}



from dataclasses import dataclass

from telebox.bot.types.type import Type


@dataclass(unsafe_hash=True)
class ChatPhoto(Type):
    small_file_id: str
    small_file_unique_id: str
    big_file_id: str
    big_file_unique_id: str



ANDORRA = "AD"
UNITED_ARAB_EMIRATES = "AE"
AFGHANISTAN = "AF"
ANTIGUA_AND_BARBUDA = "AG"
ANGUILLA = "AI"
ALBANIA = "AL"
ARMENIA = "AM"
ANGOLA = "AO"
ANTARCTICA = "AQ"
ARGENTINA = "AR"
AMERICAN_SAMOA = "AS"
AUSTRIA = "AT"
AUSTRALIA = "AU"
ARUBA = "AW"
ALAND_ISLANDS = "AX"
AZERBAIJAN = "AZ"
BOSNIA_AND_HERZEGOVINA = "BA"
BARBADOS = "BB"
BANGLADESH = "BD"
BELGIUM = "BE"
BURKINA_FASO = "BF"
BULGARIA = "BG"
BAHRAIN = "BH"
BURUNDI = "BI"
BENIN = "BJ"
SAINT_BARTHELEMY = "BL"
BERMUDA = "BM"
BRUNEI_DARUSSALAM = "BN"
BOLIVIA = "BO"
BONAIRE = "BQ"
BRAZIL = "BR"
BAHAMAS = "BS"
BHUTAN = "BT"
BOUVET_ISLAND = "BV"
BOTSWANA = "BW"
BELARUS = "BY"
BELIZE = "BZ"
CANADA = "CA"
COCOS_ISLANDS = "CC"
DEMOCRATIC_REPUBLIC_OF_CONGO = "CD"
CENTRAL_AFRICAN_REPUBLIC = "CF"
CONGO = "CG"
SWITZERLAND = "CH"
COTE_DIVOIRE = "CI"
COOK_ISLANDS = "CK"
CHILE = "CL"
CAMEROON = "CM"
CHINA = "CN"
COLOMBIA = "CO"
COSTA_RICA = "CR"
CUBA = "CU"
CABO_VERDE = "CV"
CURACAO = "CW"
CHRISTMAS_ISLAND = "CX"
CYPRUS = "CY"
CZECHIA = "CZ"
GERMANY = "DE"
DJIBOUTI = "DJ"
DENMARK = "DK"
DOMINICA = "DM"
DOMINICAN_REPUBLIC = "DO"
ALGERIA = "DZ"
ECUADOR = "EC"
ESTONIA = "EE"
EGYPT = "EG"
WESTERN_SAHARA = "EH"
ERITREA = "ER"
SPAIN = "ES"
ETHIOPIA = "ET"
FINLAND = "FI"
FIJI = "FJ"
FALKLAND_ISLANDS = "FK"
MICRONESIA = "FM"
FAROE_ISLANDS = "FO"
FRANCE = "FR"
GABON = "GA"
UNITED_KINGDOM_OF_GREAT_BRITAIN_AND_NORTHERN_IRELAND = "GB"
GRENADA = "GD"
GEORGIA = "GE"
FRENCH_GUIANA = "GF"
GUERNSEY = "GG"
GHANA = "GH"
GIBRALTAR = "GI"
GREENLAND = "GL"
GAMBIA = "GM"
GUINEA = "GN"
GUADELOUPE = "GP"
EQUATORIAL_GUINEA = "GQ"
GREECE = "GR"
SOUTH_GEORGIA_AND_THE_SOUTH_SANDWICH_ISLANDS = "GS"
GUATEMALA = "GT"
GUAM = "GU"
GUINEA_BISSAU = "GW"
GUYANA = "GY"
HONG_KONG = "HK"
HEARD_ISLAND_AND_MCDONALD_ISLANDS = "HM"
HONDURAS = "HN"
CROATIA = "HR"
HAITI = "HT"
HUNGARY = "HU"
INDONESIA = "ID"
IRELAND = "IE"
ISRAEL = "IL"
ISLE_OF_MAN = "IM"
INDIA = "IN"
BRITISH_INDIAN_OCEAN_TERRITORY = "IO"
IRAQ = "IQ"
IRAN = "IR"
ICELAND = "IS"
ITALY = "IT"
JERSEY = "JE"
JAMAICA = "JM"
JORDAN = "JO"
JAPAN = "JP"
KENYA = "KE"
KYRGYZSTAN = "KG"
CAMBODIA = "KH"
KIRIBATI = "KI"
COMOROS = "KM"
SAINT_KITTS_AND_NEVIS = "KN"
NORTH_KOREA = "KP"
SOUTH_KOREA = "KR"
KUWAIT = "KW"
CAYMAN_ISLANDS = "KY"
KAZAKHSTAN = "KZ"
LAOS = "LA"
LEBANON = "LB"
SAINT_LUCIA = "LC"
LIECHTENSTEIN = "LI"
SRI_LANKA = "LK"
LIBERIA = "LR"
LESOTHO = "LS"
LITHUANIA = "LT"
LUXEMBOURG = "LU"
LATVIA = "LV"
LIBYA = "LY"
MOROCCO = "MA"
MONACO = "MC"
MOLDOVA = "MD"
MONTENEGRO = "ME"
SAINT_MARTIN = "MF"
MADAGASCAR = "MG"
MARSHALL_ISLANDS = "MH"
NORTH_MACEDONIA = "MK"
MALI = "ML"
MYANMAR = "MM"
MONGOLIA = "MN"
MACAO = "MO"
NORTHERN_MARIANA_ISLANDS = "MP"
MARTINIQUE = "MQ"
MAURITANIA = "MR"
MONTSERRAT = "MS"
MALTA = "MT"
MAURITIUS = "MU"
MALDIVES = "MV"
MALAWI = "MW"
MEXICO = "MX"
MALAYSIA = "MY"
MOZAMBIQUE = "MZ"
NAMIBIA = "NA"
NEW_CALEDONIA = "NC"
NIGER = "NE"
NORFOLK_ISLAND = "NF"
NIGERIA = "NG"
NICARAGUA = "NI"
NETHERLANDS = "NL"
NORWAY = "NO"
NEPAL = "NP"
NAURU = "NR"
NIUE = "NU"
NEW_ZEALAND = "NZ"
OMAN = "OM"
PANAMA = "PA"
PERU = "PE"
FRENCH_POLYNESIA = "PF"
PAPUA_NEW_GUINEA = "PG"
PHILIPPINES = "PH"
PAKISTAN = "PK"
POLAND = "PL"
SAINT_PIERRE_AND_MIQUELON = "PM"
PITCAIRN = "PN"
PUERTO_RICO = "PR"
PALESTINE = "PS"
PORTUGAL = "PT"
PALAU = "PW"
PARAGUAY = "PY"
QATAR = "QA"
REUNION = "RE"
ROMANIA = "RO"
SERBIA = "RS"
RUSSIAN_FEDERATION = "RU"
RWANDA = "RW"
SAUDI_ARABIA = "SA"
SOLOMON_ISLANDS = "SB"
SEYCHELLES = "SC"
SUDAN = "SD"
SWEDEN = "SE"
SINGAPORE = "SG"
SAINT_HELENA_ASCENSION_AND_TRISTAN_DA_CUNHA = "SH"
SLOVENIA = "SI"
SVALBARD_AND_JAN_MAYEN = "SJ"
SLOVAKIA = "SK"
SIERRA_LEONE = "SL"
SAN_MARINO = "SM"
SENEGAL = "SN"
SOMALIA = "SO"
SURINAME = "SR"
SOUTH_SUDAN = "SS"
SAO_TOME_AND_PRINCIPE = "ST"
EL_SALVADOR = "SV"
SINT_MAARTEN = "SX"
SYRIAN_ARAB_REPUBLIC = "SY"
ESWATINI = "SZ"
TURKS_AND_CAICOS_ISLANDS = "TC"
CHAD = "TD"
FRENCH_SOUTHERN_TERRITORIES = "TF"
TOGO = "TG"
THAILAND = "TH"
TAJIKISTAN = "TJ"
TOKELAU = "TK"
TIMOR_LESTE = "TL"
TURKMENISTAN = "TM"
TUNISIA = "TN"
TONGA = "TO"
TURKIYE = "TR"
TRINIDAD_AND_TOBAGO = "TT"
TUVALU = "TV"
TAIWAN = "TW"
TANZANIA = "TZ"
UKRAINE = "UA"
UGANDA = "UG"
UNITED_STATES_MINOR_OUTLYING_ISLANDS = "UM"
UNITED_STATES_OF_AMERICA = "US"
URUGUAY = "UY"
UZBEKISTAN = "UZ"
HOLY_SEE = "VA"
SAINT_VINCENT_AND_THE_GRENADINES = "VC"
VENEZUELA = "VE"
BRITISH_VIRGIN_ISLANDS = "VG"
UNITED_STATES_VIRGIN_ISLANDS = "VI"
VIETNAM = "VN"
VANUATU = "VU"
WALLIS_AND_FUTUNA = "WF"
SAMOA = "WS"
YEMEN = "YE"
MAYOTTE = "YT"
SOUTH_AFRICA = "ZA"
ZAMBIA = "ZM"
ZIMBABWE = "ZW"



ARTICLE = "article"
PHOTO = "photo"
GIF = "gif"
MPEG4_GIF = "mpeg4_gif"
VIDEO = "video"
AUDIO = "audio"
VOICE = "voice"
DOCUMENT = "document"
LOCATION = "location"
VENUE = "venue"
CONTACT = "contact"
GAME = "game"
STICKER = "sticker"



from dataclasses import dataclass
from typing import Optional

from telebox.bot.types.type import Type
from telebox.bot.types.types.message_entity import MessageEntity


@dataclass(unsafe_hash=True)
class InputTextMessageContent(Type):
    message_text: str
    parse_mode: Optional[str] = None
    entities: Optional[list[MessageEntity]] = None
    disable_web_page_preview: Optional[bool] = None



from __future__ import annotations
from typing import TYPE_CHECKING

from telebox.dispatcher.filters.event_filter import AbstractEventFilter
from telebox.dispatcher.enums.event_type import EventType
if TYPE_CHECKING:
    from telebox.dispatcher.typing import Event


class NoneFilter(AbstractEventFilter):

    def get_event_types(self) -> set[EventType]:
        return set(EventType)

    def get_value(self, event: Event) -> bool:
        return True

    def check_value(self, value: bool) -> bool:
        return value



MENTION = "mention"
HASHTAG = "hashtag"
CASHTAG = "cashtag"
BOT_COMMAND = "bot_command"
URL = "url"
EMAIL = "email"
PHONE_NUMBER = "phone_number"
BOLD = "bold"
ITALIC = "italic"
UNDERLINE = "underline"
STRIKETHROUGH = "strikethrough"
SPOILER = "spoiler"
CODE = "code"
PRE = "pre"
TEXT_LINK = "text_link"
TEXT_MENTION = "text_mention"
CUSTOM_EMOJI = "custom_emoji"
